'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// The middleware is kept as simple as possible.
// All favicon operations are delegated to a function below.


exports.default = function (favicoOptions) {
  // Detect if this middleware is being used without being 'preloaded',
  // by being passed a store instead of favicoOptions
  if (favicoOptions && typeof favicoOptions.getState === 'function') {
    console.error('\n      redux-favicon middleware not preloaded!\n      You need to first call reduxFavicon with its configuration to initialize it, THEN pass it to createStore.\n\n      See https://github.com/joshwcomeau/redux-favicon#setup');
  }

  var favicon = favicoIntegration(favicoOptions);

  return function (store) {
    return function (next) {
      return function (action) {
        // Ignore actions that don't tweak the favicon
        if (!action.meta || typeof action.meta.favicon === 'undefined') {
          return next(action);
        }

        favicon.update(action.meta.favicon, function (err) {
          if (err) console.warn(err);

          return next(action);
        });
      };
    };
  };
};

var _favico = require('favico.js');

var _favico2 = _interopRequireDefault(_favico);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultFavicoOptions = { animation: 'slide' };
var favicoEnumValues = ['increment', 'decrement', 'reset'];

// This integration communicates directly with Favico.js to set the badge.
function favicoIntegration() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultFavicoOptions;

  // Create a new Favico integration.
  // Initially this was going to be a singleton object, but I realized there
  // may be cases where you want several different types of notifications.
  // The middleware does not yet support multiple instances, but it should
  // be easy to add if there's demand :)
  //
  // Not using the constructor pattern, because it obfuscates JS' prototypal
  // nature, and there's no need for inheritance here, so this is equivalent.
  var favico = new _favico2.default(options);

  return {
    currentVal: 0,

    update: function update(value, callback) {
      if (typeof value === 'number') {
        // Don't allow non-integer values
        if (value % 1 !== 0) {
          return callback('\n            Warning: Favico not affected.\n            You provided a floating-point value: ' + value + '.\n            You need to provide an integer, or a keyword value.\n\n            See https://github.com/joshwcomeau/redux-favicon#troubleshooting for more information.\n          ');
        }

        this.currentVal = value;
      } else if (typeof value === 'string') {
        switch (value.toLowerCase()) {
          case 'increment':
            this.currentVal++;break;
          case 'decrement':
            this.currentVal--;break;
          case 'reset':
            this.currentVal = 0;break;
          default:
            return callback('\n              Warning: Favico not affected.\n              You provided a string value: ' + value + '.\n              The only strings we accept are: ' + favicoEnumValues.join(', ') + '.\n\n              See https://github.com/joshwcomeau/redux-favicon#troubleshooting for more information.\n            ');
        }
      } else {
        // Annoyingly, istanbul won't give me 100% coverage unless all possible
        // typeof values are checked. It is not possible for all types to make
        // it to this check; `undefined` aborts earlier.
        /* istanbul ignore next */
        var provided_type = typeof value === 'undefined' ? 'undefined' : _typeof(value);

        return callback('\n          Warning: Favico provided an illegal type.\n          You provided a a value of type: ' + provided_type + '.\n          We only accept integers or strings.\n\n          See https://github.com/joshwcomeau/redux-favicon#troubleshooting for more information.\n        ');
      }

      // Don't allow negative numbers
      this.currentVal = this.currentVal < 0 ? 0 : this.currentVal;

      // Set the 'badge' to be our derived value.
      // The favico.js library will show it if it's truthy, hide it if falsy.
      favico.badge(this.currentVal);

      return callback();
    }
  };
}
module.exports = exports['default'];